@@ -318,6 +318,12 @@
   (tree-go-to t :end)
   (make 'label))
 
+(tm-define (focus-label t)
+  (:require (section-context? t))
+  (and-with p (tree-up t)
+    (and (tm-func? p 'concat)
+         (focus-search-label p))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Routines for lists, enumerations and description
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -347,7 +353,8 @@
       (with lab (inside-which (list-tag-list))
 	(cond ((in? lab (itemize-tag-list)) (make 'item))
 	      ((in? lab (enumerate-tag-list)) (make 'item))
-	      ((in? lab (description-tag-list)) (make 'item*))))))
+	      ((in? lab (description-tag-list)) (make 'item*))
+              (else (make 'item))))))
 
 (tm-define (kbd-enter t shift?)
   (:require (list-context? t))
@@ -357,6 +364,13 @@
   (:require (tree-is? t 'item*))
   (go-end-of 'item*))
 
+(tm-define (focus-label t)
+  (:require (or (list-context? t) (tree-is? t 'bib-list)))
+  (and-with doc (tree-down t)
+    (and (tree-is? doc 'document)
+         (and-with par (tree-down doc)
+           (focus-search-label par)))))
+
 (tm-define (numbered-context? t)
   (:require (or (itemize-context? t) (enumerate-context? t)))
   #t)
@@ -398,7 +412,7 @@
     (insert ins)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; Inserting formulas
+;; Formulas
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (tm-define (make-equation)
@@ -416,6 +430,21 @@
   (make 'eqnarray*)
   (temp-proof-fix))
 
+(tm-define (focus-label t)
+  (:require (tree-is? t 'equation))
+  (focus-list-search-label (tree-children t)))
+
+(define (down-to-row t)
+  (cond ((not (tree? t)) #f)
+        ((tree-is? t 'row) t)
+        ((tree-in? t '(document tformat table)) (down-to-row (tree-down t)))
+        (else #f)))
+
+(tm-define (focus-label t)
+  (:require (tree-in? t '(eqnarray eqnarray*)))
+  (and-with row (down-to-row (tree-down t))
+    (focus-search-label row)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Routines for inserting miscellaneous content
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -504,6 +533,11 @@
   (tree-go-to t :last :start)
   (make 'dueto))
 
+(tm-define (focus-label t)
+  (:require (tree-in? t (enunciation-tag-list)))
+  (and (== (tree-arity t) 1)
+       (focus-search-label (tree-ref t 0))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Editing algorithms
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -583,6 +617,31 @@
 	  (tree-insert! t (- (tree-arity t) 1) '((document "")))
 	  (tree-go-to t (- (tree-arity t) 2) :start)))))
 
+(tm-define (focus-label t)
+  (:require (algorithm-context? t))
+  (focus-list-search-label (tree-children t)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Possible to use a custom note symbol
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define (detached-note-context? t)
+  (tree-in? t (detached-note-tag-list)))
+
+(tm-define (auto-note-context? t)
+  (tree-in? t (auto-note-tag-list)))
+
+(tm-define (custom-note-context? t)
+  (tree-in? t (custom-note-tag-list)))
+
+(tm-define (note-toggle-custom t)
+  (let* ((l (symbol-toggle-number (tree-label t)))
+         (c (tree-children t)))
+    (if (auto-note-context? t)
+        (tree-insert! t (tree-arity t) (list "<dag>"))
+        (tree-remove! t (- (tree-arity t) 1) 1))
+    (tree-assign-node! t l)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Possible to change the title of titled environments
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -650,11 +709,19 @@
            (tree-set! t `(,l ,(tree-ref t 0)))
            (tree-go-to t 0 :end)))))
 
+(tm-define (parameter-choice-list var)
+  (:require (in? var (list "padding-above" "padding-below")))
+  (list "0fn" "0.5fn" "1fn" "1.5fn" "2fn" :other))
+
 (tm-define (customizable-parameters t)
   (:require (tree-in? t '(padded padded-titled)))
   (list (list "padding-above" "Above")
         (list "padding-below" "Below")))
 
+(tm-define (parameter-choice-list var)
+  (:require (in? var (list "overlined-sep" "underlined-sep")))
+  (list "0sep" "0.5sep" "1sep" "1.5sep" "2sep" :other))
+
 (tm-define (customizable-parameters t)
   (:require (tree-in? t '(overlined overlined-titled)))
   (list (list "padding-above" "Above")
@@ -678,29 +745,37 @@
   (:require (tree-in? t '(framed framed-titled)))
   (list (list "padding-above" "Above")
         (list "padding-below" "Below")
-        (list "framed-vsep" "Inner")
-        (list "framed-hsep" "Indentation")
         (list "framed-color" "Color")))
 
+(tm-define (parameter-choice-list var)
+  (:require (in? var (list "ornament-hpadding" "ornament-vpadding")))
+  (list "0spc" "0.5spc" "1spc" "1.5spc" "2spc" :other))
+
+(tm-define (parameter-choice-list var)
+  (:require (in? var (list "ornament-border")))
+  (list "0ln" "0.5ln" "1ln" "2ln" "3ln" "4ln" "5ln" :other))
+
 (tm-define (customizable-parameters t)
   (:require (tree-in? t '(ornamented decorated)))
   (list (list "padding-above" "Above")
         (list "padding-below" "Below")
         (list "ornament-vpadding" "Inner")
         (list "ornament-hpadding" "Indentation")
+        (list "ornament-shape" "Shape")
         (list "ornament-color" "Color")
-        (list "ornament-shape" "Shape")))
+        (list "ornament-border" "Border")))
 
 (tm-define (customizable-parameters t)
-  (:require (tree-is? t '(ornamented-titled decorated-titled)))
+  (:require (tree-in? t '(ornamented-titled decorated-titled)))
   (list (list "padding-above" "Above")
         (list "padding-below" "Below")
         (list "ornament-vpadding" "Inner")
         (list "ornament-hpadding" "Indentation")
-        (list "ornament-color" "Color")
         (list "ornament-shape" "Shape")
-        (list "ornament-extra-color" "Title color")
-        (list "ornament-title-style" "Title style")))
+        (list "ornament-color" "Color")
+        (list "ornament-border" "Border")
+        (list "ornament-title-style" "Title style")
+        (list "ornament-extra-color" "Title color")))
 
 (tm-define (customizable-parameters t)
   (:require (tree-is? t 'ornament))
@@ -720,6 +795,35 @@
         (list "ornament-hpadding" "Horizontal padding")
         (list "ornament-vpadding" "Vertical padding")))
 
+(tm-define (parameter-choice-list var)
+  (:require (in? var (list "frame-hpadding" "frame-vpadding")))
+  (list "0tab" "0.5tab" "1tab" "1.5tab" "2tab" :other))
+
+(tm-define (parameter-choice-list var)
+  (:require (in? var (list "frame-thickness")))
+  (list "0.2" "0.5" "1" "1.5" "2" "3" "4" "5" :other))
+
+(tm-define (customizable-parameters t)
+  (:require (tree-in? t (art-frame-tag-list)))
+  (list (list "frame-thickness" "Thickness")
+        (list "frame-recolor" "Recolor")
+        (list "frame-hpadding" "Horizontal padding")
+        (list "frame-vpadding" "Vertical padding")))
+
+(tm-define (parameter-choice-list var)
+  (:require (in? var (list "shadow-elevation")))
+  (list "0.2" "0.5" "1" "1.5" "2" "3" "4" "5" :other))
+
+(tm-define (parameter-choice-list var)
+  (:require (in? var (list "shadow-plain")))
+  (list "false" "true"))
+
+(tm-define (customizable-parameters t)
+  (:require (tree-in? t (shadow-tag-list)))
+  (list (list "shadow-elevation" "Elevation")
+        (list "shadow-recolor" "Recolor")
+        (list "shadow-plain" "Plain")))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Floating objects and environments
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -730,6 +834,9 @@
 (tm-define (footnote-context? t)
   (tree-in? t '(footnote wide-footnote)))
 
+(tm-define (figure-context? t)
+  (tree-in? t (figure-tag-list)))
+
 (tm-define (float-or-footnote-context? t)
   (tree-in? t '(float wide-float footnote wide-footnote)))
 
@@ -844,3 +951,7 @@
   (if (cursor-at-anchor?)
       (go-to-float)
       (go-to-anchor)))
+
+(tm-define (focus-label t)
+  (:require (or (footnote-context? t) (figure-context? t)))
+  (focus-list-search-label (tree-children t)))
